//+------------------------------------------------------------------+
//|                                              CompareFunction.mqh |
//|                                    2019-2021, dimitri pecheritsa |
//|                             https://www.mql5.com/en/users/dmipec |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| a universal overloaded compare method for all standard mql types |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| compare bool                                                     |
//+------------------------------------------------------------------+
int Compare(const bool x,const bool y)
  {
   return (x>y) ? 1 : (x<y) ? -1 : 0;
  }
//+------------------------------------------------------------------+
//| compare char                                                     |
//+------------------------------------------------------------------+
int Compare(const char x,const char y)
  {
   return (x>y) ? 1 : (x<y) ? -1 : 0;
  }
//+------------------------------------------------------------------+
//| compare uchar                                                    |
//+------------------------------------------------------------------+
int Compare(const uchar x,const uchar y)
  {
   return (x>y) ? 1 : (x<y) ? -1 : 0;
  }
//+------------------------------------------------------------------+
//| compare short                                                    |
//+------------------------------------------------------------------+
int Compare(const short x,const short y)
  {
   return (x>y) ? 1 : (x<y) ? -1 : 0;
  }
//+------------------------------------------------------------------+
//| compare ushort                                                   |
//+------------------------------------------------------------------+
int Compare(const ushort x,const ushort y)
  {
   return (x>y) ? 1 : (x<y) ? -1 : 0;
  }
//+------------------------------------------------------------------+
//| compare color                                                    |
//+------------------------------------------------------------------+
int Compare(const color x,const color y)
  {
   return (x>y) ? 1 : (x<y) ? -1 : 0;
  }
//+------------------------------------------------------------------+
//| compare int                                                      |
//+------------------------------------------------------------------+
int Compare(const int x,const int y)
  {
   return (x>y) ? 1 : (x<y) ? -1 : 0;
  }
//+------------------------------------------------------------------+
//| compare uint                                                     |
//+------------------------------------------------------------------+
int Compare(const uint x,const uint y)
  {
   return (x>y) ? 1 : (x<y) ? -1 : 0;
  }
//+------------------------------------------------------------------+
//| compare datetime                                                 |
//+------------------------------------------------------------------+
int Compare(const datetime x,const datetime y)
  {
   return (x>y) ? 1 : (x<y) ? -1 : 0;
  }
//+------------------------------------------------------------------+
//| compare long                                                     |
//+------------------------------------------------------------------+
int Compare(const long x,const long y)
  {
   return (x>y) ? 1 : (x<y) ? -1 : 0;
  }
//+------------------------------------------------------------------+
//| compare ulong                                                    |
//+------------------------------------------------------------------+
int Compare(const ulong x,const ulong y)
  {
   return (x>y) ? 1 : (x<y) ? -1 : 0;
  }
//+------------------------------------------------------------------+
//| compare float                                                    |
//+------------------------------------------------------------------+
int Compare(const float x,const float y,int digits=13)
  {
   double min=pow(10,-fabs(digits));
   return (((fabs(x-y)<min))? 0 : ((x-y>=min))? 1 : -1);
  }
//+------------------------------------------------------------------+
//| compare double                                                   |
//+------------------------------------------------------------------+
int Compare(const double x,const double y,int digits=13)
  {
   double min=pow(10,-fabs(digits));
   return (((fabs(x-y)<min))? 0 : ((x-y>=min))? 1 : -1);
  }
//+------------------------------------------------------------------+
//| compare string                                                   |
//+------------------------------------------------------------------+
int Compare(const string x,const string y)
  {
   return (x>y) ? 1 : (x<y) ? -1 : 0;
  }
//+------------------------------------------------------------------+
//| interface comparable                                             |
//+------------------------------------------------------------------+
template<typename T>
interface IComparable 
  {
   int       Compare(T value);
  };
//+------------------------------------------------------------------+
//| compare custom object by casting to a comparable object          |
//+------------------------------------------------------------------+
template<typename T>
int Compare(T x,T y)
  {
//--- try to convert to comparable object
   IComparable<T>*comparable=dynamic_cast<IComparable<T>*>(x);
   return
      (comparable)?
//--- use specified compare method
      comparable.Compare(y):
//--- unknown compare function
      int("nan");
  }
//+------------------------------------------------------------------+
//| interface comparer                                               |
//+------------------------------------------------------------------+
template<typename T>
interface IComparer
  {
   int       Compare(T x,T y);
  };
//+------------------------------------------------------------------+
//| compare custom object with external comparer                     |
//+------------------------------------------------------------------+
template<typename T>
int Compare(T x,T y,IComparer* comparer)
  {
   return comparer.Compare(x,y);
  }
//+------------------------------------------------------------------+
